Method: Libuv::Handle#unref
- Defined in:
- lib/libuv/handle.rb
#unref ⇒ Object
Public: Decrement internal ref counter for the handle on the loop, useful to stop loop even when there are outstanding open handles
Returns self
39 40 41 42 |
# File 'lib/libuv/handle.rb', line 39 def unref return if @closed ::Libuv::Ext.unref(handle) end |